#!/bin/sh

# SVN : $Id: Start_LuaUPnP.sh 16506 2016-03-31 21:20:56Z nicolas.veloz $

#Copyright (C) 2008 Mi Casa Verde, Inc., a Nevada Corporation
#                    www.micasaverde.com
#           1 - 702 - 4879770 / 866 - 966 - casa
#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
#without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

#set -x

cmh_log_file="/tmp/log/cmh/LuaUPnP.log"
file_STOP_CMH="/var/lock/cmh.stop"
file_PID_START_CMH="/var/run/start_cmh.pid"
tmp_file_PID_START_CMH=${file_PID_START_CMH}.$$

zw_upgrade_file="/tmp/zwave.upgrade"

#zw_fw_path="/etc/cmh-zwfw"
#zw_fw_upd_inprogress_file="$zw_fw_path/.zwave_fw_upgrading"

log_file="/tmp/log.Init-LuaUPnP"
log() {
    echo "`date +%F_%T`|$$|START_LU| $*" >> $log_file
}

log "===BEGIN==="

#read debug parameters
DEBUG_COREDUMPS=$(awk -F '=' '/^DEBUG_COREDUMPS/ {print $2}' /etc/cmh/cmh.conf 2>/dev/null)
DEBUG_GDBSERVER=$(awk -F '=' '/^DEBUG_GDBSERVER/ {print $2}' /etc/cmh/cmh.conf 2>/dev/null)
[[ "$DEBUG_GDBSERVER" == 1 ]] && {
	GDBSERVER=$(which gdbserver) || log "ERROR : Please install gdbserver with : opkg update && opkg install gdbserver";
	GDBPORT="127.0.0.1:$(awk -F'=' '/^PORT/ {print $2}' /etc/cmh-ra/cmh-ra.conf 2>/dev/null)"
}

# Only 1 can run at a time, do a lock.  Write our pid to a file
[ -d /var/run ] || mkdir -p /var/run
echo $$ > $tmp_file_PID_START_CMH

GotLock="0"

# Try 3 times to get the lock
for i in $(seq 1 3); do
	ln -s $tmp_file_PID_START_CMH $file_PID_START_CMH
	if [[ $? == "0" ]] && [[ $(cat $file_PID_START_CMH) == $$ ]]; then
		log "LOCK: Got lock"
		GotLock="1"
		break
	else
		pid=$(cat $file_PID_START_CMH)
		if [[ -z "$pid" ]] || [[ ! -d /proc/$pid ]]; then
			log "LOCK: PID: $pid finished.  Clearing the lock."
			rm -rf $file_PID_START_CMH || /bin/true
		else
			log "LOCK: Failed to get lock"
			sleep 1
		fi
	fi
done

if [[ "$GotLock" == "0" ]] ; then
	log "LOCK: Allready running. Quitting"
	rm $tmp_file_PID_START_CMH
	exit 1
fi

#rm -f /var/run/cmh.stop

mkdir -p /tmp/log/cmh
mkdir -p /tmp/buffer # a temporary directory to store stuff that's automatically cleared on each reload
#export MiOS own library path
export LD_LIBRARY_PATH="/usr/lib/cmh:/mios/usr/lib/cmh:/rom/usr/lib/cmh"
export LUA_PATH="./?.lua;/usr/share/lua/?.lua;/usr/share/lua/?/init.lua;/usr/lib/lua/?.lua;/usr/lib/lua/?/init.lua;/etc/cmh-ludl/?.lua;/etc/cmh-lu/?.lua"


printf "02 $(date +%F\ %T) - LuaUPnP boot\n" >> $cmh_log_file

kill $(ps ax | grep 'Start_serproxy.sh.$\|Start_serproxy.sh$' | awk '{print $1}')
killall serproxy
sleep 1
kill -9 $(ps ax | grep 'Start_serproxy.sh.$\|Start_serproxy.sh$' | awk '{print $1}')
killall -9 serproxy
/usr/bin/Start_serproxy.sh &
log "Started SERPROXY in background with PID:$!"

if [[ "$DEBUG_COREDUMPS" == 1 ]]; then
	#enable core dumps
	log "*** Enabling Core Dumps ***"
	ulimit -c unlimited
	log "*** WARNING be sure to have a usb stick mounted else we could fill /tmp ***"
	#set coredumps to go into /tmp
	echo "/tmp/log/cmh/core-%e-%s-%p-%t" > /proc/sys/kernel/core_pattern
else
	#disable core dumps
	ulimit -c 0
fi

while [[ ! -f /var/run/cmh.stop ]]; do

	if [[ -f $zw_upgrade_file ]]; then
		#Luup Engine set the file to upgrade the firmware the last time it run
		#we need to read the file and get the desired firmware version
		#call the UpdateZWaveFirmware script and wait it here until finish
		/usr/bin/UpdateZWaveFirmware.sh "$(cat $zw_upgrade_file)" 1>>$log_file 2>&1
		exit_code=$?
		printf "\n\n\033[7;36m%s\033[1;00m\n\n" "$(date +%F\ %T) - UpdateZWaveFirmware Terminated with Exit Code: $exit_code" | tee -a $cmh_log_file >> $log_file
		rm $zw_upgrade_file
	fi

	#if UpdateZWaveFirmware is running we should wait it to finish before starting
#	while [[ -f $zw_fw_upd_inprogress_file ]] && [[ -d /proc/$(cat $zw_fw_upd_inprogress_file 2>/dev/null) ]]; do
#		printf "\n\n\033[7;36m%s\033[1;00m\n\n" "$(date +%F\ %T) - Waiting UpdateZWaveFirmware to finish" | tee -a $cmh_log_file >> $log_file
#		sleep 10
#	done

	if [[ "$DEBUG_GDBSERVER" == 1 ]]; then
		#start LuaUPnP for remote debugging
                set_led.sh on service
		printf "\n\n\033[7;36m%s\033[1;00m\n\n" "$(date +%F\ %T) - START LuaIPnP for remote gdb debugging using : $GDBSERVER" | tee -a $cmh_log_file >> $log_file
		$GDBSERVER $GDBPORT /usr/bin/LuaUPnP 1>>$log_file 2>&1
		exit_code=$?
    set_led.sh off service
	else
		#start LuaUPnP normally
                set_led.sh on service
		/usr/bin/LuaUPnP 1>>$log_file 2>&1
		exit_code=$?
    set_led.sh off service
	fi

	printf "\n\n\033[7;36m%s\033[1;00m\n\n" "$(date +%F\ %T) - LuaUPnP Terminated with Exit Code: $exit_code" | tee -a $cmh_log_file >> $log_file
	if [[ -f /tmp/battery.shutdown ]]; then
		printf "\n\n\033[7;36m%s\033[1;00m\n\n" "$(date +%F\ %T) - LuaUPnP forcing halt" | tee -a $cmh_log_file >> $log_file
		halt
	fi
	if [[ -f /tmp/log/cmh/signal.log ]]; then
		printf "\n\n\033[7;36m%s\033[1;00m\n\n" "$(date +%F\ %T) - LuaUPnP crash" >> $cmh_log_file
		cat /tmp/log/cmh/signal.log >> $cmh_log_file
	fi

	echo 0 > /tmp/zwave_state
	kill -HUP `pidof NetworkMonitor | awk '{print $1}'`  # turn off the Z-Wave light

	# sometimes some processes aren't killed
	echo "---------------exited-------------" >> /tmp/log/cmh/LuaUPnP.log
	ps axf >> /tmp/log/cmh/LuaUPnP.log
	killall LuaUPnP
	sleep 1
	echo "---------------after sleep-------------" >> /tmp/log/cmh/LuaUPnP.log
	ps axf >> /tmp/log/cmh/LuaUPnP.log
	killall -9 LuaUPnP
	echo "---------------after kill -9-------------" >> /tmp/log/cmh/LuaUPnP.log
	ps axf >> /tmp/log/cmh/LuaUPnP.log
	killall serproxy ser2net # be sure we start this whenever LuaUPnP exits
	rm -rf /tmp/buffer/*
done

log "Start_LuaUPnP.sh done" >> $log_file
log "===END==="
