#!/bin/sh /etc/rc.common

# SVN : $Id: mios_fix_time.sh 14178 2015-04-14 13:41:46Z florin $

# NOTE: Since between reboots the unit can change its timezone, we should store/restore the time only in UTC.
DATE_OPT='-u'
DATE_SET_FMT='+%Y%m%d%H%M'
DATE_SHOW_FMT='+%Y-%m-%d %H:%M:%S'
DATE_EPOCH=$(date "$DATE_SET_FMT" -d "2018-10-01 00:00:00") # The epoch MUST be before 2010, because LuaUPnP disables running scenes if the date is older than 2010.

#set -x

START=115
STOP=95

DATETIME_FILE="/etc/cmh/datetime"
TIME_SYNCED_FILE="/tmp/log.ntpclient"
DIFF_DATETIME=1 # How many minutes it takes to startup.

log_file="/tmp/log.fix_datetime"
log() {
	printf "%s -[%s]- %s\n" "$(date +%F_%T)" "$$" "$*" >> $log_file
}


start()
{
	log "START"
	# Restore time only on boards that don't have a built-in RTC.
	if [ -e /dev/rtc0 ]; then
		log "This unit has a built in clock. Restore system time from RTC($(hwclock -u -r)): $(hwclock -u -s && echo OK || echo FAILED)"
		log "Time restored from RTC: $(date -R)"
	# Restore time only if it was synced from internet.
	elif [ -s "$DATETIME_FILE" ]; then
		local lastDatetime=$(cat "$DATETIME_FILE" 2>>$log_file)
		local currentDatetime=$(date "$DATE_OPT" "$DATE_SET_FMT")
		log "Last saved datetime=$lastDatetime UTC, current datetime=$currentDatetime UTC, timezone=$(cat /etc/TZ 2>>$log_file)"
		# Set a new datetime only if the current one is not accurate.
		# If we just synced the datetime from the Internet skip it, else set it to the datetime from file.
		if [ ! -f "$TIME_SYNCED_FILE" ]; then
			local newDatetime=$((lastDatetime + DIFF_DATETIME))
			log "Current time is not accurate. Set it to: $(date "$DATE_OPT" "$DATE_SHOW_FMT" -d "$newDatetime") UTC"
			# Set datetime.
			date "$DATE_OPT" "$DATE_SET_FMT" -s "$newDatetime" 1>>$log_file 2>&1
			log "Time restored from file: $(date -R)"
		else
			log "Unit just synced time from internet. Restoring from file not required"
		fi
		# Be sure to remove the datetime file, so in case of force reboot, to not sync time to a very old time.
		log "Remove datetime file: $DATETIME_FILE"
		rm -f "$DATETIME_FILE" 2>>$log_file
	# In other cases set the datetime to a predefined epoch.
	else
		log "Previous datetime hasn't been saved in $DATETIME_FILE, and unit doesn't have built-in clock"
		log "Keep System fixed time"
#		date "$DATE_OPT" "$DATE_SET_FMT" -s "$DATE_EPOCH" 1>/dev/null 2>>$log_file
#		if [ $? -eq 0 ]; then
#			log "Datetime successfully set"
#		else
#			log "ERROR: Failed to set datetime"
#		fi
	fi

	log "date:    $(date -R)" >>$log_file
	log "TZ:      $(cat /etc/TZ 2>/dev/null)" >>$log_file
	log "TZ-full: $(cat /etc/TZ-full 2>/dev/null)" >>$log_file
	log "END"
}


stop()
{
	# Store the current datetime only if it was synced from internet.
	if [ -f "$TIME_SYNCED_FILE" ]; then
		local currentDatetime=$(date "$DATE_OPT" "$DATE_SET_FMT")
		log "Store current datetime: $(date "$DATE_OPT" "$DATE_SHOW_FMT" -d "$currentDatetime") UTC"
		echo "$currentDatetime" > $DATETIME_FILE
		[ -e /dev/rtc0 ] && log "Update RTC($(hwclock -u -r)) from system time: $(hwclock -u -w && echo OK || echo FAILED)"
	else
		log "Datetime wasn't synced from Internet. We won't store current time in file or RTC"
	fi
}
