#!/usr/bin/haserl
<?
# Copyright (C) 2009 MiOS, Ltd., a Hong Kong Corporation
#                     www.micasaverde.com
#            1 - 702 - 4879770 / 866 - 966 - casa
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.


# ABOUT THIS SCRIPT
# This script passes the received requests to the appropriate server and returns the response headers and parameters unmodified.
# It receives either the ServerTYPE (which maps in /etc/cmh/servers.conf) and the ServerURL
# or only the url, and returns the response headers and output from the server unmodified.

begin_time=$(date +%s)

MIOS_CONSTANTS_FILE="$DOCUMENT_ROOT/../mios_constants.sh"
[ -f "$MIOS_CONSTANTS_FILE" ] && source "$MIOS_CONSTANTS_FILE" || echo "FILE-NOT-FOUND: mios_constants.sh"

log_to="file"; log_file="$MIOS_LOGS_PATH/log.cgi-proxy"; log_as="proxy"
[ -f "$MIOS_FUNCTIONS_FILE" ] && source "$MIOS_FUNCTIONS_FILE" || echo "FILE-NOT-FOUND: functions.sh"

PROXYSH_DEBUG=0

PROXY_TMP_DIR="$MIOS_TMP_PATH/proxy"
RESPONSE_HEADER_FILE="$PROXY_TMP_DIR/header_$$.txt"
RESPONSE_BODY_FILE="$PROXY_TMP_DIR/body_$$.txt"
RESPONSE_ERROR_FILE="$PROXY_TMP_DIR/stderr_$$.txt"

# If this is '1', we won't save the output to a temporary file in /tmp,
# but we'll return what we receive as we receive it (streaming mode).
g_unbuffered=0


mkdir -p "$PROXY_TMP_DIR" 2>/dev/null;
clean_tmp_files() {
	rm -f "$RESPONSE_HEADER_FILE" "$RESPONSE_BODY_FILE" "$RESPONSE_ERROR_FILE" 2>/dev/null
}
clean_tmp_files
if [ $PROXYSH_DEBUG == 0 ]; then
	trap 'clean_tmp_files' INT QUIT EXIT KILL
fi

Dlog() {
	[ $PROXYSH_DEBUG == 1 ] && log "$*"
}

decode_arguments() {
	# Decode base64 passwords
	# CurlParams="$CurlParams -F \"$paramName=$paramValEsc\""
	# FORM_encoded=param1,param2,...,paramN
	# If we don't have any base64 encoded arguments, no processing is necessary.
	if [ -n "$FORM_encoded" ]; then
		local IFS=','
		for param in $FORM_encoded; do
			b64enc_val=$(env | grep "^FORM_${param}=" | cut -d '=' -f '2-')
			decoded=$(echo $b64enc_val | openssl enc -base64 -d)
			export FORM_${param}="$decoded"
		done
	fi
}

######################################################################################
#                                        MAIN                                        #
######################################################################################
log_START "proxy.sh"

Dlog "ENV:\n$(env | sort)"

# Check for code injection.
for var in $(env | grep '^FORM_' | grep -Ev '(Old|New)Password' | awk -F '=' '{ print $1 }'); do
	if eval "echo \$$var | grep -q '\\\$(\|\`'"; then
		log "ERROR: Found invalid characters in arguments: $(env | grep '^FORM_')"
		echo "ERROR: Invalid characters in arguments"
		exit 1
	fi
done
decode_arguments

if [ -n "$FORM_ServerTYPE" ] && [ -n "$FORM_ServerURL" ]; then
	log "Start for server: $FORM_ServerTYPE"
	Dlog "on url: $FORM_ServerURL"
	serverType=$(printf '%s' "$FORM_ServerTYPE" | tr 'A-Z' 'a-z')
	alt=0
	if [ "${serverType%_alt}" != "$serverType" ]; then
		serverType=${serverType%_alt}
		alt=1
	fi
	Server=$(get_server "$serverType" $alt 2>/dev/null)
	if [ -z "$Server" ]; then
		echo "ERROR: No server of type $serverType defined"
		log_DIE "No server of type $serverType defined."
	fi
	ServerAlt=$(get_server "$serverType" $((1-alt)) 2>/dev/null)
	if [ "$ServerAlt" == "$Server" ]; then
		ServerAlt=""
	fi
	URL="https://${Server}/${FORM_ServerURL}"
	if [ -n "$ServerAlt" ]; then
		URL_alt="https://${ServerAlt}/${FORM_ServerURL}"
	fi
elif [ -n "$FORM_url" ]; then
	URL=$FORM_url
	log "Start for URL: ${URL:0:50}"
	Dlog "Full URL: $URL"
	server=$(echo "$URL" | awk -F'/' '/^http/{print $3; next} {print $1}')
	if ! validate_server "$server"; then
		echo "ERROR: Server not allowed"
		exit 1
	fi
elif [ -n "$FORM_url_unbuf" ]; then
	URL=$FORM_url_unbuf
	g_unbuffered=1
	log "Start unbuffered for URL: ${URL:0:50}"
	Dlog "Full URL: $URL"
	server=$(echo "$URL" | awk -F'/' '/^http/{print $3; next} {print $1}')
	if ! validate_server "$server"; then
		echo "ERROR: Server not allowed"
		exit 1
	fi
else
	echo "ERROR: Incorrectly called"
	# FORM_name are the parameters
	# HTTP_name are the headers
	log_ERROR "Incorrectly called with parameters:\n$(env | grep '^FORM_\|^HTTP_')"
	exit 1
fi

URL_woParams=$(echo "$URL" | awk -F '?' '{ print $1 }')
[ $PROXYSH_DEBUG == 1 ] && URL_Params=$(echo "$URL" | awk -F '?' '{ print $2 }')

# Do we have MMS authentication headers defined?
MMSHeaders=""
[ -n "$HTTP_MMSAUTH" ] && MMSHeaders="$MMSHeaders -H \"MMSAuth: $HTTP_MMSAUTH\""
[ -n "$HTTP_MMSAUTHSIG" ] && MMSHeaders="$MMSHeaders -H \"MMSAuthSig: $HTTP_MMSAUTHSIG\""
[ -n "$HTTP_MMSSESSION" ] && MMSHeaders="$MMSHeaders -H \"MMSSession: $HTTP_MMSSESSION\""

if [ -n "$FORM_method" ]; then
	RestMethod=$(echo "$FORM_method" | tr a-z A-Z)
else
	RestMethod="GET"
fi

# Eliminate 100-continue header for multiform post parameters (-F)
ExtraHeaders="-H “Expect:"-H “Content-Length: 0""

# Read all POST parameters names.
PostParams=$(env | awk -F'=' '/^FORM_/ {print $1}' |\
	sed '/^FORM_url\|^FORM_ServerURL\|^FORM_ServerTYPE\|^FORM_method/d; s/^FORM_//g')

# Compose curl POST parameters
CurlParams=''
for paramName in $PostParams; do
	eval "paramVal=\$FORM_$paramName"
	paramValEsc=$(printf "%s" "$paramVal" | sed -e 's/\\/\\\\/g; s/\"/\\"/g; s/`/\\`/g; s/\$/\\\$/g')
	Dlog "$paramName=$paramVal / $paramValEsc"
	CurlParams="$CurlParams -F \"$paramName=$paramValEsc\""
done

# Escape some special characters, to prevent code injection.
URL=$(echo "$URL" | sed 's/`/%60/g; s/!/%21/g; s/\$/%24/g; s/"/%22/g; s/;/%3b/g')

RestHeaders="${MMSHeaders:+$MMSHeaders }${ExtraHeaders:+$ExtraHeaders}"
RunCurl="curl -k -sS -L $RestHeaders -X $RestMethod $CurlParams --url \"$URL\""
if [ $g_unbuffered == 0 ]; then
	RunCurl="$RunCurl --max-time 30 -D \"$RESPONSE_HEADER_FILE\" -o \"$RESPONSE_BODY_FILE\""
else
	RunCurl="$RunCurl -D - -o -"
fi
RunCurl="$RunCurl 2>$RESPONSE_ERROR_FILE"

log "Connect to url: ($RestMethod) $URL_woParams"
Dlog "Headers: $RestHeaders"
[ $PROXYSH_DEBUG == 1 ] && Dlog "URL parameters: $URL_Params"
Dlog "Parameters: $CurlParams"

Dlog "Run CMD: eval $RunCurl"
eval "$RunCurl"
curlExitCode=$?
log "curl exit code: $curlExitCode"

if [ $g_unbuffered == 0 ]; then
	HttpStatus=$(cat "$RESPONSE_HEADER_FILE" | awk '/HTTP\/1\.[01]/ { code = $2 } END { print code }' | tr -d '\r\n')
	MMSResponse=$(cat "$RESPONSE_HEADER_FILE" | awk '/MMSResponse:/ { print $2 }' | tr -d '\r\n')
	log "HTTP status: $HttpStatus, MMSResponse: $MMSResponse"

	# Process curl exit code.
	if [ $curlExitCode -eq 0 ]; then
		log_SUCCESS "Request OK"
	elif [ -n "$ServerAlt" ] && { [ $curlExitCode -eq 7 ] || [ "$HttpStatus" == 500 ] || [ "$MMSResponse" == 500 ]; } then
		log "Try on the other server"
		URL=$(echo "$URL_alt" | sed 's/`/%60/g; s/!/%21/g; s/\$/%24/g')
		RunCurl="curl -k -sS -L $RestHeaders -X $RestMethod $CurlParams --url \"$URL\" -D \"$RESPONSE_HEADER_FILE\" -o \"$RESPONSE_BODY_FILE\" 2>$RESPONSE_ERROR_FILE"
		Dlog "Run CMD: eval $RunCurl"
		eval "$RunCurl"
		curlExitCode=$?
		if [ $curlExitCode -eq 0 ]; then
			log_SUCCESS "Request OK"
		else
			log_ERROR "Failed to connect with exit code: $curlExitCode to URL: $URL_woParams with method: $RestMethod"
		fi
	else
		log_ERROR "Failed to connect with exit code: $curlExitCode to URL: $URL_woParams with method: $RestMethod"
	fi
	Dlog "GotHeaders: \n$(cat $RESPONSE_HEADER_FILE 2>/dev/null | tr '\r' '\n')"
	Dlog "GotBody: \n$(cat $RESPONSE_BODY_FILE 2>/dev/null)"
	Dlog "GotError: \n$(cat $RESPONSE_ERROR_FILE 2>/dev/null)"

	### SEND OUTPUT ###
	cat "$RESPONSE_HEADER_FILE" "$RESPONSE_BODY_FILE" 2>/dev/null
fi
###

end_time=$(date +%s)
log "END proxy.sh in $((end_time - begin_time)) seconds"
?>
